@@ -434,6 +434,8 @@ On Windows, we use SSPI authentication to ensure the same (by pg_regress
 WAL archiving can be enabled on this node by passing the keyword parameter
 has_archiving => 1. This is disabled by default.
 
+Data checksums can be forced off by passing data_checksums => 1.
+
 postgresql.conf can be set up for replication by passing the keyword
 parameter allows_streaming => 'logical' or 'physical' (passing 1 will also
 suffice for physical replication) depending on type of replication that
@@ -456,6 +458,12 @@ sub init
 	$params{allows_streaming} = 0 unless defined $params{allows_streaming};
 	$params{has_archiving}    = 0 unless defined $params{has_archiving};
 
+        # This should override user-supplied initdb options.
+        if ($params{data_checksums})
+        {
+                push @{ $params{extra} }, '--data-checksums';
+        }
+
 	mkdir $self->backup_dir;
 	mkdir $self->archive_dir;
 
@@ -2759,6 +2767,37 @@ sub pg_recvlogical_upto
 
 =pod
 
+=item $node->corrupt_page_checksum(self, file, page_offset)
+
+Intentionally corrupt the checksum field of one page in a file.
+The server must be stopped for this to work reliably.
+
+The file name should be specified relative to the cluster datadir.
+page_offset had better be a multiple of the cluster's block size.
+
+=cut
+
+sub corrupt_page_checksum
+{
+        my ($self, $file, $page_offset) = @_;
+        my $pgdata = $self->data_dir;
+        my $pageheader;
+
+        open my $fh, '+<', "$pgdata/$file" or die "open($file) failed: $!";
+        binmode $fh;
+        sysseek($fh, $page_offset, 0) or die "sysseek failed: $!";
+        sysread($fh, $pageheader, 24) or die "sysread failed: $!";
+        # This inverts the pd_checksum field (only); see struct PageHeaderData
+        $pageheader ^= "\0\0\0\0\0\0\0\0\xff\xff";
+        sysseek($fh, $page_offset, 0) or die "sysseek failed: $!";
+        syswrite($fh, $pageheader) or die "syswrite failed: $!";
+        close $fh;
+
+        return;
+}
+
+=pod
+
 =back
 
 =cut
